/*
reorg.c Version 0.1.0  Hash reorganize database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: reorg filename */

#include <bthash.h>

#define MAXRCDS 9999999

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Reorganize a hash database\n");
   fprintf(stderr,"Usage %s options filename\n", pgm);
   fprintf(stderr,"Options:\n");
   fprintf(stderr,"   -t integer for total records\n");
   fprintf(stderr,"   -f real    for free space percent\n");
   fprintf(stderr,"Where total records "
      "is 1 - %d\n", MAXRCDS);
   fprintf(stderr,"      free space    "
      "is 0.0 - 100.0\n");
   fprintf(stderr,"Example: %s "
      "-t 100 -f 10.0 "
      "myfile.hsh\n", pgm);
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i;
   int opnerr;
   int rslt;
   int totrcds;
   int arglmt;
   double freespc;   /* 0.0 - 100.0 percent */
   hashfmt *hshhdr;
   char dbname[256];
   if (argc == 2 || argc == 4 || argc == 6);
   else putstx(*argv);
   totrcds = 0;
   freespc = 0.0;
   arglmt = argc - 1;
   i = 1;
   while (i < arglmt)
      {
      rslt = strcmp(*(argv+i),"-t");
      if (!rslt)
	 {
	 totrcds = atoi(*(argv+i+1));
	 i = i + 2;
	 continue;
	 } /* total records */
      rslt = strcmp(*(argv+i),"-f");
      if (!rslt)
	 {
	 freespc = atoi(*(argv+i+1));
	 i = i + 2;
	 continue;
	 } /* record length */
      fprintf(stderr,"Invalid parameter %s\n",
	 *(argv+i));
      putstx(*argv);
      } /* for each parm */
   strcpy(dbname,*(argv+i));

   if (totrcds < 0 || totrcds > MAXRCDS)
      {
      fprintf(stderr,"reorg: total_records "
	 "out of range\n");
      putstx(*argv);
      } /* invalid totrcds */
   if (freespc < 0.0 || freespc > 100.0)
      {
      fprintf(stderr,"reorg: free_space out of range\n");
      putstx(*argv);
      } /* invalid freespc */


   hshhdr = (hashfmt *) hopen(dbname,UPDT,LOCK,&opnerr);
   if (hshhdr == NULL)
      {
      fprintf(stderr,"reorg: hopen "
	 "error %d\n", opnerr);
      exit(1);
      } /* open err */
   /* process starts here */

   rslt = hreorg(hshhdr,totrcds,freespc);
   if (rslt || hshhdr->reason)
      {
      fprintf(stderr,"reorg: hreorg "
	 "error %d reason %d\n",
	 rslt, hshhdr->reason);
      } /* if reorg err */

   /**********************************************************/
   hclose(hshhdr);
   return(0);
   } /* main */
